#!/usr/bin/perl -w
use warnings;
use strict;
use XML::LibXML;

sub load_rcat;
sub parse_journal($);

#------------------------------------------------------------------------------
# Globals
my %rcat;			# req catalogue for a subsystem: a hash of (id, text) elems.
my $total = 0;		# total number of reqs for a subsystem
my $covered = 0;	# number of covered reqs for a subsystem

my $all_total = 0;	# total number of reqs for all subsystems
my $all_covered = 0;# number of covered reqs for all subsystems

my %reqs;			# a hash of checked (covered) requirement IDs (key=id, value is insignificant for now).
# $reqs{$id} == 0 if this req was not found in the rcats, 1 otherwise.

my $txt_list="";	# hrefs to files with lists of not checked reqs.
#------------------------------------------------------------------------------
# get T2C root dir
my $t2c_suite_root = $ENV{T2C_SUITE_ROOT};
if (not $t2c_suite_root)
{
	print "T2C_SUITE_ROOT is undefined.\n";
	$t2c_suite_root = ".";
}
else
{
	print "T2C_ROOT is ", $t2c_suite_root, "\n";
}

chdir $t2c_suite_root or die "Unable to change current directory to $t2c_suite_root.\n";
#------------------------------------------------------------------------------

# Parse journal file, load the list of covered REQs
my $jdir = shift @ARGV;
$jdir = "0001e" unless ($jdir);

my $jpath = "results/$jdir/journal";
die "The $jpath file is missing.\n" unless (-f $jpath);

print "Journal file: $jpath\n";
parse_journal($jpath);

#------------------------------------------------------------------------------
# Open report file and write a header.
open REPF, "> results/$jdir/coverage-summary.htm" or die $!;
print REPF "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n";
print REPF "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n\n<head>\n";
print REPF "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />\n<title>Coverage report - summary</title></head>\n\n";
print REPF "<body>\n<h2>Requirement id coverage (summary)</h2>\n";
print REPF "<table width=\"80%\"  border=\"1\" cellpadding=\"0\" cellspacing=\"0\">\n<tr align=\"center\">\n";
print REPF "<td><b>Subsystem</b></td>\n<td><b>All requirements</b></td>\n<td><b>Checked requirements</b></td>\n<td><b>Coverage (by req ids)</b></td>\n</tr>\n\n";

#------------------------------------------------------------------------------
# Load requirement catalogues and prepare reports
my @subsystems = glob("*-t2c");
print "Subsystems:\n";

for my $subs (@subsystems)
{
	my $rcat_dir = $subs."/reqs";
	print "$subs,\trcat dir: $rcat_dir\n";
	unless (-d $rcat_dir)
	{
		print "WARNING: requirement catalogue directory is missing:\n\t$rcat_dir.\n";
		next;
	}
	
	# Remove "-t2c" suffix
	$subs =~ s/-t2c//;
	
	my $out_file = "results/$jdir/not_checked-$subs.txt";
	open OUTF, "> $out_file" or die $!;
	
	# get a list of the req catalogue XML files
	my @rcat_xml = glob("$rcat_dir/*.xml");
	unless (@rcat_xml)
	{
		print "WARNING: no files in the requirement catalogue dir.\n";
		next;
	}

	%rcat = (); # clear the hash 
	$total = 0;
	$covered = 0;
	
	# load requirement catalogues from XML files into a single global hash 
	for (@rcat_xml)
	{
		unless (-f $_)
		{
			print "WARNING: $_ is not a plain file.\n";
			next;
		}
		
		load_rcat;
	} # for (@rcat_xml)
	
	$total = keys(%rcat);
	$all_total += $total;
	
	# Remove checked reqs from the catalogue.
	# The remaining ones are those that were not covered in this test run.
	for my $req (keys(%reqs))
	{
		if (exists($rcat{$req}))
		{
			delete $rcat{$req};
			$reqs{$req} = 1;	# mark this req as "found"
		}
	}
	
	$covered = $total - keys(%rcat);
	$all_covered += $covered;
		
	print "Total number of reqs: $total, checked: $covered.\n";
	print "\n";
	
	my $coverage = int($covered/$total * 10000)/100;
	
	print OUTF "Total number of reqs: $total, checked: $covered, coverage: ", $coverage, "%\n\n";
	print OUTF "The following requirements were not checked:\n\n";
	for my $req (sort(keys(%rcat)))
	{
		print OUTF "{$req}\n$rcat{$req}\n\n";
	}
	
	print REPF "<tr align=\"center\">\n<td>$subs</td>\n";
	print REPF "<td>$total</td>\n";
	print REPF "<td>$covered</td>\n";
	print REPF "<td>$coverage", "%</td>\n";
	print REPF "</tr>\n";
	
	$txt_list = $txt_list."<li><a href=\"not_checked-$subs.txt\">not checked - $subs</a></li>\n";
	
} # for my $subs (@subsystems)

# Check if some reqs are missing from the catalogues.
my $missing_file = "results/$jdir/missing_reqs.txt";
open MISSF, "> $missing_file" or die $!;

print MISSF "The following requirements are missing from the catalogues:\n\n";
my $nmiss = 0;
for my $id (sort(keys(%reqs)))
{
	if ($reqs{$id} == 0)
	{
		++$nmiss;
		print MISSF "{$id}\n";
	}
}

print "Missing reqs: $nmiss\n";

# Output the rest of the report
print REPF "<tr align=\"center\">\n<td><b>Total</b></td>\n";
print REPF "<td><b>$all_total</b></td>\n";
print REPF "<td><b>$all_covered</b></td>\n";
print REPF "<td><b>", int($all_covered/$all_total * 10000)/100, "%</b></td>\n";
print REPF "</tr>\n";

print REPF "</table><br/><br/>\n\n";
print REPF "Click on appropriate link below to see which requirements were not checked in this test run.\n";
print REPF "<ul>\n";
print REPF $txt_list;
print REPF "</ul>\n\n";

print REPF "Checked requirements missing from the catalogues: $nmiss (";
print REPF "<a href=\"missing_reqs.txt\">list...</a>).\n\n";

print REPF "</body>\n</html>\n";

#------------------------------------------------------------------------------
sub load_rcat
{
	# [NB] Do not increment $total here: a req can be defined in more than one file.
	
	my $parser = new XML::LibXML;
	my $doc = $parser->parse_file($_);
	
	for my $record ( $doc->findnodes( '//req' )) 
	{
		my $id = $record->findvalue('@id');
		
		if ($id =~ /\A\s*\z/)
		{
			print "WARNING: Empty or missing req ID in $_\n";
			next;
		}
		
		if ($record->textContent =~ /\A\s*\z/)
		{
			print "WARNING: Empty requirement text for ID {$id}.\n";
		}
		
		$rcat{$id} = $record->textContent;
		$rcat{$id} =~ s/\A\s+//;
	}	
}

#------------------------------------------------------------------------------
sub parse_journal($)
{
	#my $jpath = shift;
	open(JRNL, shift) or die $!;
	
	while (<JRNL>)
	{
		my @block = split(/\|/);
		my $num = @block;
		
		if (($num >= 3) and ($block[0] eq "520"))
		{
			# found a message ($block[2]), now parse it 
			
			my $str = $block[2];

			if ($str =~ s/Checked requirements:\s*//)
			{
				#print "req checked\n";
				$str =~ s/\s+//g;
				
				my @ids = split(/;/, $str);
				for my $id (@ids)
				{
					$reqs{$id} = 0; # not found in rcats by default
				}
			}
			elsif ($str =~ /\{([\w|\.]+\.\d+)\}/)
			{
				$reqs{$1} = 0;
			}
			else
			{
				# do nothing: no req id in this line
			}

		} # end if
	}
	print "The journal has been parsed, found ", scalar keys(%reqs), " checked requirements.\n";
}

#------------------------------------------------------------------------------

